# -*- coding: utf-8 -*-

from rest_framework import serializers

from ... import models
from thx.api.serializers import (
    ThuxListSerializerMixin, ThuxRetrieveSerializerMixin, ThuxCreateSerializerMixin,
    ThuxUpdateSerializerMixin, ThuxPartialUpdateSerializerMixin, ThuxStatusSerializerMixin,
    ThuxEnableSerializerMixin, ThuxDisableSerializerMixin
)


# ADDRESS SCOPE
class AddressScopeSerializer(serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope'
        model = models.AddressScope
        fields = '__all__'


class AddressScopeListSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope_list'
        model = models.AddressScope
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class AddressScopeRetrieveSerializer(ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope_retrieve'
        model = models.AddressScope
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class AddressScopeCreateSerializer(ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope_create'
        model = models.AddressScope
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressScopeListSerializer(instance, context=self.context).data


class AddressScopeUpdateSerializer(ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope_update'
        model = models.AddressScope
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressScopeRetrieveSerializer(instance, context=self.context).data


class AddressScopePartialUpdateSerializer(ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope_partial_update'
        model = models.AddressScope
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressScopeRetrieveSerializer(instance, context=self.context).data


class AddressScopeEnableSerializer(ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope_enable'
        model = models.AddressScope
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance

    def to_representation(self, instance):
        return AddressScopeRetrieveSerializer(instance, context=self.context).data


class AddressScopeDisableSerializer(ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope_disable'
        model = models.AddressScope
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance

    def to_representation(self, instance):
        return AddressScopeRetrieveSerializer(instance, context=self.context).data


class AddressScopeLightSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_scope_light'
        model = models.AddressScope
        fields = ('id', 'status', 'get_status_display',)

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


# ADDRESS
class AddressSerializer(serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address'
        model = models.Address
        fields = '__all__'


class AddressListSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_list'
        model = models.Address
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class AddressRetrieveSerializer(ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_retrieve'
        model = models.Address
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class AddressCreateSerializer(ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_create'
        model = models.Address
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressListSerializer(instance, context=self.context).data


class AddressUpdateSerializer(ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_update'
        model = models.Address
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressRetrieveSerializer(instance, context=self.context).data


class AddressPartialUpdateSerializer(ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_partial_update'
        model = models.Address
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressRetrieveSerializer(instance, context=self.context).data


class AddressEnableSerializer(ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_enable'
        model = models.Address
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance

    def to_representation(self, instance):
        return AddressRetrieveSerializer(instance, context=self.context).data


class AddressDisableSerializer(ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_disable'
        model = models.Address
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance

    def to_representation(self, instance):
        return AddressRetrieveSerializer(instance, context=self.context).data


class AddressLightSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_light'
        model = models.Address
        fields = ('id', 'status', 'get_status_display',)

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


# ADDRESS COORDINATE
class AddressCoordinateSerializer(serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate'
        model = models.AddressCoordinate
        fields = '__all__'


class AddressCoordinateListSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate_list'
        model = models.AddressCoordinate
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class AddressCoordinateRetrieveSerializer(ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate_retrieve'
        model = models.AddressCoordinate
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []


class AddressCoordinateCreateSerializer(ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate_create'
        model = models.AddressCoordinate
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressCoordinateListSerializer(instance, context=self.context).data


class AddressCoordinateUpdateSerializer(ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate_update'
        model = models.AddressCoordinate
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressCoordinateRetrieveSerializer(instance, context=self.context).data


class AddressCoordinatePartialUpdateSerializer(ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate_partial_update'
        model = models.AddressCoordinate
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []

    def to_representation(self, instance):
        return AddressCoordinateRetrieveSerializer(instance, context=self.context).data


class AddressCoordinateEnableSerializer(ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate_enable'
        model = models.AddressCoordinate
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance

    def to_representation(self, instance):
        return AddressCoordinateRetrieveSerializer(instance, context=self.context).data


class AddressCoordinateDisableSerializer(ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate_disable'
        model = models.AddressCoordinate
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance

    def to_representation(self, instance):
        return AddressCoordinateRetrieveSerializer(instance, context=self.context).data


class AddressCoordinateLightSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_address_address_coordinate_light'
        model = models.AddressCoordinate
        fields = ('id', 'status', 'get_status_display',)

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'staff':
            return fields
        return []
